/*
 * Copyright (C) 2021-2022 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __STA_H__
#define __STA_H__

#include "common.h"
#include "libpowerfip.h"

void tst_pwrfip_var_handler(struct pwrfip_node *node,
                struct pwrfip_var *var, struct pwrfip_event *evt);
void tst_pwrfip_ba_state_handler(struct pwrfip_node *node,
                struct pwrfip_event *evt);
void tst_pwrfip_error_handler(struct pwrfip_node *node,
                enum pwrfip_error_code code);
void tst_pwrfip_reset_handler(struct pwrfip_node *node);
void tst_pwrfip_sync_rw_op_handler(struct pwrfip_node *node,
    struct pwrfip_var *var, struct pwrfip_event *evt);
void print_node_infos(struct pwrfip_node *node);

#define TST_MAX_STA_COUNT 2
#define TST_BA_MCYCLE_MID_US 2500 /* 2.5ms */
#define TST_BA_MCYCLE_US 5000 /* 5ms (default cycle time) */
static struct pwrfip_var_cfg tst_var_cfg[TST_MAX_STA_COUNT][TST_MAX_VAR_COUNT] = {
    /* station 0 */
    {
        /* [0x3800] (prod) */
        {
            .type = PWRFIP_VAR_TYPE_PROD,
            .id = 0x3800,
            .prod.payload_bsz = 8,
            .prod.flags = \
                /* enable prod status */
                PWRFIP_VAR_FLAGS_REFRESH,
            .prod.refreshment_ustime = (TST_BA_MCYCLE_US + TST_BA_MCYCLE_MID_US),
            .prod.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
        /* [0x3801] (cons) */
        {
            .type = PWRFIP_VAR_TYPE_CONS,
            .id = 0x3801,
            .cons.payload_bsz = 8,
            .cons.flags = \
                /* enable prod status */
                PWRFIP_VAR_FLAGS_REFRESH | \
                /* enable promptness checking */
                PWRFIP_VAR_FLAGS_PROMPT | \
                /* enable pdu + len bytes checking */
                PWRFIP_VAR_FLAGS_CHK_PDU_LEN,
            .cons.promptness_ustime = (TST_BA_MCYCLE_US + TST_BA_MCYCLE_MID_US),
            .cons.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
        /* [0xFF00] (sync) - end mcycle id_dat  */
        /* note: here we use it for both r/w operations */
        {
            .type = PWRFIP_VAR_TYPE_SYNC,
            .id = 0xFF01,
            .pwrfip_var_handler = tst_pwrfip_sync_rw_op_handler,
        },
    },
    /* station 1 */
    {
        /* [0x3801] (prod) */
        {
            .type = PWRFIP_VAR_TYPE_PROD,
            .id = 0x3801,
            .prod.payload_bsz = 8,
            .prod.flags = \
                /* enable prod status */
                PWRFIP_VAR_FLAGS_REFRESH,
            .prod.refreshment_ustime = (TST_BA_MCYCLE_US + TST_BA_MCYCLE_MID_US),
            .prod.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
        /* [0x3800] (cons) */
        {
            .type = PWRFIP_VAR_TYPE_CONS,
            .id = 0x3800,
            .cons.payload_bsz = 8,
            .cons.flags = \
                /* enable prod status */
                PWRFIP_VAR_FLAGS_REFRESH | \
                /* enable promptness checking */
                PWRFIP_VAR_FLAGS_PROMPT | \
                /* enable pdu + len bytes checking */
                PWRFIP_VAR_FLAGS_CHK_PDU_LEN,
            .cons.promptness_ustime = (TST_BA_MCYCLE_US + TST_BA_MCYCLE_MID_US),
            .cons.evt_type = PWRFIP_EVT_TYPE_NONE,
            .pwrfip_var_handler = NULL,
        },
        /* [0xFF00] (sync) - end mcycle id_dat  */
        /* note: here we use it for both r/w operations */
        {
            .type = PWRFIP_VAR_TYPE_SYNC,
            .id = 0xFF01,
            .pwrfip_var_handler = tst_pwrfip_sync_rw_op_handler,
        },
    }
};

/* ba periodic requests lists [for periodic mcycle wind] */
/* => ping list */
#define TST_BA_PER_PING_REQ_COUNT 2
static struct pwrfip_ba_request tst_ba_per_ping_req[TST_BA_PER_PING_REQ_COUNT] = {
    {
        .type = PWRFIP_BA_ID_DAT,
        .id = 0x3800,
    },
    {
        .type = PWRFIP_BA_ID_DAT,
        .id = 0xFF00, /* pure sync event for ping operation  */
    },
};
/* => pong list */
#define TST_BA_PER_PONG_REQ_COUNT 2
static struct pwrfip_ba_request tst_ba_per_pong_req[TST_BA_PER_PONG_REQ_COUNT] = {
    {
        .type = PWRFIP_BA_ID_DAT,
        .id = 0x3801,
    },
    {
        .type = PWRFIP_BA_ID_DAT,
        .id = 0xFF01, /* pure sync event for pong operation  */
                      /* note: not used here, but can be used by external nodes */
    },
};

#define TST_MAX_BA_MCYCLE_WIND_COUNT 4
#define TST_BA_MCYCLE_WIND_COUNT 4
static struct pwrfip_ba_wind_cfg tst_ba_wind_cfg[TST_MAX_BA_MCYCLE_COUNT][TST_MAX_BA_MCYCLE_WIND_COUNT] = {
    /* ba mcycle 1 */
    {
        /* 1 - periodic variable window (ping var) */
        {
            .type = PWRFIP_BA_WIND_PER,
            .per.req_cnt = TST_BA_PER_PING_REQ_COUNT,
            .per.req_list = &tst_ba_per_ping_req[0],
        },
        /* 2 - resync wait window */
        {
            .type = PWRFIP_BA_WIND_WAIT,
            .wait.end_ustime = TST_BA_MCYCLE_MID_US,
            .wait.is_silent = 0, /* fill window with padding frames */
            .wait.is_ext_resync = 0, /* internal resynchronization */
        },
        /* 3 - periodic variable window (pong var) */
        {
            .type = PWRFIP_BA_WIND_PER,
            .per.req_cnt = TST_BA_PER_PONG_REQ_COUNT,
            .per.req_list = &tst_ba_per_pong_req[0],
        },
        /* 4 - resync wait window */
        {
            .type = PWRFIP_BA_WIND_WAIT,
            .wait.end_ustime = TST_BA_MCYCLE_US,
            .wait.is_silent = 0, /* fill window with padding frames */
            .wait.is_ext_resync = 0, /* internal resynchronization */
        },
    },
};

static struct pwrfip_ba_mcycle_cfg tst_ba_mcycle_cfg[TST_MAX_BA_MCYCLE_COUNT] = {
    /* ba mcycle 1 */
    {
        .wind_cnt = TST_BA_MCYCLE_WIND_COUNT,
        .wind_list = &tst_ba_wind_cfg[0][0],
    },
};

#define PWRFIP_BITRATE_RANGE (_PWRFIP_BITRATE_MAX - _PWRFIP_BITRATE_MIN)

/* turn around time (default) in us, according to the bitrate */
static const uint32_t tr_ustime[PWRFIP_BITRATE_RANGE] = {
    424, /* 424us@31.25Kbps */
    30, /* 30us@1Mbps */
    30, /* 30us@2.5Mbps */
    32, /* 32us@5Mbps */
    32, /* 32us@12.5Mbps */
    32, /* 32us@25Mbps */
};

/* silence time (default) in us, according to the bitrate */
static const uint32_t ts_ustime[PWRFIP_BITRATE_RANGE] = {
    4096, /* 4096us@31.25Kbps */
    150, /* 150us@1Mbps */
    96, /* 96us@2.5Mbps */
    92, /* 92us@5Mbps */
    92, /* 92us@12.5Mbps */
    92, /* 92us@25Mbps */
};

static const char *pwrfip_node_state_str[_PWRFIP_NODE_STATE_MAX] = {
    /* PWRFIP_NODE_STATE_INITIAL */
    "initial",
    /* PWRFIP_NODE_STATE_LOADED */
    "loaded",
    /* PWRFIP_NODE_STATE_READY */
    "ready",
    /* PWRFIP_NODE_STATE_RUNNING */
    "running",
};

static const char *pwrfip_ba_state_str[_PWRFIP_BA_STATE_MAX] = {
    /* PWRFIP_BA_STATE_INITIAL */
    "initial",
    /* PWRFIP_BA_STATE_READY */
    "ready",
    /* PWRFIP_BA_STATE_STARTING */
    "starting",
    /* PWRFIP_BA_STATE_IDLE */
    "idle",
    /* PWRFIP_BA_STATE_RUNNING */
    "running",
};

static const char *pwrfip_node_op_str[_PWRFIP_NODE_OP_MAX] = {
    /* _PWRFIP_NODE_OP_UNKNOWN */
    "unknown",
    /* PWRFIP_NODE_OP_WAIT_RX_RP_FRM */
    "rx rp frame",
    /* PWRFIP_NODE_OP_WAIT_TX_RP_FRM */
    "tx rp frame",
    /* PWRFIP_NODE_OP_WAIT_RX_ID_FRM */
    "rx id frame",
    /* PWRFIP_NODE_OP_WAIT_TX_ID_FRM */
    "tx id frame",
};

static const char *pwrfip_ba_wind_str[_PWRFIP_BA_WIND_TYPE_MAX] = {
    /* _PWRFIP_BA_WIND_TYPE_NONE */
    "none",
    /* PWRFIP_BA_WIND_PER */
    "periodic",
    /* PWRFIP_BA_WIND_APER_VAR */
    "aper. var",
    /* PWRFIP_BA_WIND_APER_MSG */
    "aper. msg",
    /* PWRFIP_BA_WIND_WAIT */
    "wait",
};

static const char *pwrfip_var_err_str[_PWRFIP_VAR_ERR_MAX] = {
    /* _PWRFIP_VAR_OK */
    "no error",
    /* PWRFIP_VAR_ID_UNKNOWN */
    "unknown var id",
    /* PWRFIP_VAR_NOT_PRODUCING */
    "not a producing var",
    /* PWRFIP_VAR_NOT_CONSUMING */
    "not a consuming var",
    /* PWRFIP_VAR_TX_APER_CH_UNKNOWN */
    "unknown tx aper channel",
    /* PWRFIP_VAR_PDU_INCONSISTENT */
    "inconsistent var pdu",
    /* PWRFIP_VAR_LEN_TOO_LONG */
    "var length too long",
    /* PWRFIP_VAR_LEN_TOO_SHORT */
    "var length too short",
    /* PWRFIP_VAR_NEVER_RECEIVED */
    "var never received",
    /* PWRFIP_VAR_TX_APER_FIFO_EMPTY */
    "tx aper queue is empty",
    /* PWRFIP_VAR_TX_APER_FIFO_FULL */
    "tx aper queue is full",
    /* PWRFIP_VAR_NOT_MEANING */
    "payload not significant",
    /* PWRFIP_VAR_NOT_REFRESH */
    "payload not refreshed",
    /* PWRFIP_VAR_NOT_PROMPT */
    "payload not prompt",
    /* PWRFIP_VAR_BAD_PROMPT_PER */
    "bad reading frequency (promptness)",
    /* PWRFIP_VAR_BAD_REFRESH_PER */
    "bad writing frequency (refreshment)"
};

#endif
