/*
 * Copyright (C) 2022 EXOLIGENT (www.exoligent.com)
 * Author: Maxime Coroyer <maxime.coroyer@exoligent.com>
 */

#ifndef __OPC_RT_H__
#define __OPC_RT_H__

#include <open62541/server.h>

#define OPC_RT_QBV_OFFSET           (25 * 1000) /* Qbv offset */
#define OPC_RT_SOCKET_PRIORITY      3
#define OPC_RT_PUB_SCHED_PRIORITY   78
#define OPC_RT_SUB_SCHED_PRIORITY   81
#define OPC_RT_APP_SCHED_PRIORITY   75
#define OPC_RT_CORE_TWO             2
#define OPC_RT_CORE_THREE           3
#define OPC_RT_SUB_WKUP_PERCENTAGE  0
#define OPC_RT_APP_WKUP_PERCENTAGE  0.3
#define OPC_RT_PUB_WKUP_PERCENTAGE  0.6

struct gw_opc_rt_thread_args {
    UA_Server *server;
    void *data;
    UA_ServerCallback callback;
    UA_Duration interval_ms;
    UA_UInt64 *callback_id;
};

/* add a callback for cyclic repetition */
UA_StatusCode opc_rt_cb_add(UA_Server *srv, UA_NodeId group_id,
    UA_ServerCallback cb, void *data, UA_Double interval_ms,
    UA_DateTime *baseTime, UA_TimerPolicy timerPolicy, UA_UInt64 *cb_id);

/* change cycle repetition of the callback */
UA_StatusCode opc_rt_cb_change(UA_Server *srv, UA_NodeId group_id,
    UA_UInt64 cb_id, UA_Double interval_ms, UA_DateTime *baseTime,
    UA_TimerPolicy timerPolicy);

/* remove the callback added for cyclic repetition */
void opc_rt_cb_remove(UA_Server *srv, UA_NodeId group_id, UA_UInt64 cb_id);

UA_StatusCode opc_rt_cb_ext_datawrite(UA_Server *srv,
    const UA_NodeId *sess_id, void *sess_ctx, const UA_NodeId *node_id,
    void *node_ctx, const UA_NumericRange *range,
    const UA_DataValue *data_value);

UA_StatusCode opc_rt_cb_ext_dataread_notif(UA_Server *srv,
    const UA_NodeId *sess_id, void *sess_ctx, const UA_NodeId *node_id,
    void *node_ctx, const UA_NumericRange *range);

int opc_rt_app_start(void *args);
int opc_rt_app_stop(void *args);

#endif
